#ifndef NODES_H
#define NODES_H

#include <stdio.h>
#include <stdlib.h>

typedef struct variable_t {
    char * name;
    struct expression_t * value;
} variable_t;

typedef struct while_t {
    struct boolean_t * boolean;
    struct block_t * block;
} while_t;

typedef struct expression_t {
    struct algebraic_op_t * algebraic_op;
    double * number;
    struct variable_t * variable;
    struct derivative_t * derivative;
    struct integrate_t * integrate;
    struct evaluate_t * evaluate;
    struct negative_t * negative;
    struct parenthesis_exp_t * parenthesis_exp;

    struct expression_t * next;
} expression_t;

typedef struct function_t {
    char * name;
    expression_t * expression;
    struct domain_t * domain;
    struct taylor_t * taylor;
    struct composite_t * composite;

    struct function_t * next;
} function_t;

typedef struct statement_t {
    struct while_t * while_st;
    struct print_t * print;
    struct for_t * for_st;
    struct if_t * if_st;
    struct integrate_t * integrate;
    struct derivative_t * derivative;
    struct evaluate_t * evaluate;
    struct variable_t * variable;
    
    struct statement_t * next;
} statement_t;

typedef struct block_t {
    statement_t * statement;
} block_t;

typedef struct print_t {
    expression_t * exp;
    char * string;
} print_t;

typedef struct for_t {
    variable_t * variable_t;
    struct boolean_t * boolean;
    block_t * block;
} for_t;

typedef struct if_t {
    struct boolean_t * boolean;
    block_t * block;
    block_t * else_block;
} if_t;

typedef struct boolean_t {
    struct domain_t * domain;
    struct not_bool_t * not_bool;
    struct bool_op_t * bool_op;
} boolean_t;

typedef struct bool_op_t {
    boolean_t * left;
    boolean_t * right;
    int op;
} bool_op_t;

typedef struct not_bool_t {
    boolean_t * boolean;
} not_bool_t;

typedef struct domain_one_op_t {
    expression_t * left;
    expression_t * right;
    int oper;
} domain_one_op_t;

typedef struct domain_two_op_t {
    expression_t * left;
    expression_t * mid;
    expression_t * right;
    int oper1;
    int oper2;
} domain_two_op_t;

typedef struct domain_t {
    domain_one_op_t * domain_one;
    domain_two_op_t * domain_two;
} domain_t;

typedef struct parenthesis_exp_t {
    expression_t * expression;
} parenthesis_exp_t;

typedef struct negative_t {
    expression_t * expression;
} negative_t;

typedef struct derivative_t {
    function_t * function;
    double * value;
    double * error;
} derivative_t;

typedef struct evaluate_t {
    function_t * function;
    double * value;
} evaluate_t;

typedef struct integrate_t {
    function_t * function;
    double * value;
    double * error;

    expression_t * lvalue;
    expression_t * rvalue;
} integrate_t;

typedef struct algebraic_op_t {
    expression_t * left;
    expression_t * right;
    int op;
} algebraic_op_t;

typedef struct composite_t {
    function_t * outter_funct;
    function_t * inner_funct;
} composite_t;

typedef struct numArray_t {
    expression_t * expressions;
    size_t size;

    struct numArray_t * next;
} numArray_t;

typedef struct taylor_t {
    int num;
    function_t * function;
} taylor_t;

typedef struct id_t {
    variable_t * variable;
    function_t * function;
} id_t;

#endif