#include "../../backend/domain-specific/calculator.h"
#include "../../backend/support/logger.h"
#include "../../backend/support/nodes.h"
#include "bison-actions.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

void yyerror(const char * string) {
    LogError("Mensaje: '%s' debido a '%s' (linea %d).", string, yytext, yylineno);
}

int ProgramGrammarAction(const int value) {
    state.succeed = true;
    state.result = value;
    return value;
}

statement_t * EvaluateGrammarAction(evaluate_t * evaluate) {
    statement_t * statement = malloc(sizeof(statement_t));
    evaluate_t * evalAux = malloc(sizeof(evaluate_t));
    evalAux->function = evaluate->function;
    evalAux->value = evaluate->value;
    statement->evaluate = evalAux;
    statement->next = NULL;
    return statement;
}

statement_t * IntegrateGrammarAction(integrate_t * integrate) {
    statement_t * statement = malloc(sizeof(statement_t));
    integrate_t * intAux = malloc(sizeof(integrate_t));
    intAux->function = integrate->function;
    intAux->error = integrate->error;
    intAux->lvalue = integrate->lvalue;
    intAux->rvalue = integrate->rvalue;
    statement->integrate = intAux;
    statement->next = NULL;
    return statement;
}

statement_t * DerivativeGrammarAction(derivative_t * derivative) {
    statement_t * statement = malloc(sizeof(statement_t));
    derivative_t * derAux = malloc(sizeof(derivative_t));
    derAux->function = derivative->function;
    derAux->error = derivative->error;
    derAux->value = derivative->value;
    statement->derivative = derAux;
    statement->next = NULL;
    return statement;
}

statement_t * StatementDeclareFunctionGrammarAction(function_t * function) {
    statement_t * statement = malloc(sizeof(statement_t));
    function_t * funAux = malloc(sizeof(function_t));
    funAux->name = funAux->name;
    funAux->expression = funAux->expression;
    funAux->domain = funAux->domain;
    funAux->taylor = funAux->taylor;
    funAux->composite = funAux->composite;
    funAux->next = NULL;
    statement->evaluate = funAux;
    statement->next = NULL;
    return statement;
}

statement_t * IfGrammarAction(if_t * if_st) {
    statement_t * statement = malloc(sizeof(statement_t));
    if_t * ifAux = malloc(sizeof(if_t));
    ifAux->block = if_st->block;
    ifAux->boolean = if_st->boolean;
    statement->if_st = ifAux;
    statement->next = NULL;
    return statement;
}

statement_t * ForGrammarAction(for_t * for_st) {
    statement_t * statement = malloc(sizeof(statement_t));
    for_t * forAux = malloc(sizeof(for_t));
    forAux->block = for_st->block;
    forAux->boolean = for_st->boolean;
    forAux->variable_t = for_st->variable_t;
    statement->for_st = forAux;
    statement->next = NULL;
    return statement;
}

statement_t * WhileGrammarAction(while_t * while_st) {
    statement_t * statement = malloc(sizeof(statement_t));
    while_t * whileAux = malloc(sizeof(while_t));
    whileAux->block = while_st->block;
    whileAux->boolean = while_st->boolean;
    statement->while_st = whileAux;
    statement->next = NULL;
    return statement;
}

statement_t * PrintGrammarAction(print_t * print) {
    statement_t * statement = malloc(sizeof(statement_t));
    print_t * printAux = malloc(sizeof(print_t));
    printAux->exp = print->exp;
    printAux->string = print->string;
    statement->print = printAux;
    statement->next = NULL;
    return statement;
}

void expressionShallowCopy(expression_t * src, expression_t * dest) {
    dest->algebraic_op = src->algebraic_op;
    dest->variable = src->variable;
    dest->derivative = src->derivative;
    dest->integrate = src->integrate;
    dest->evaluate = src->evaluate;
    dest->negative = src->negative;
    dest->parenthesis_exp = src->parenthesis_exp;
    dest->number = src->number;
    dest->next = src->next;
}

void statementShallowCopy(statement_t * src, statement_t * dest) {
    dest->while_st = src->while_st;
    dest->print = src->print;
    dest->for_st = src->for_st;
    dest->if_st = src->if_st;
    dest->integrate = src->integrate;
    dest->derivative = src->derivative;
    dest->evaluate = src->evaluate;
    dest->variable = src->variable;

    dest->next = src->next;
}

void compositeShallowCopy(composite_t * src, composite_t * dest) {
    dest->outter_funct = src->outter_funct;
    dest->inner_funct = src->inner_funct;
}

expression_t * ExpressionOperatorGrammarAction(expression_t * firstExpression, int token, expression_t * secondExpression) {
    expression_t * expression = malloc(sizeof(expression_t));
    expression_t * firstExpressionAux = malloc(sizeof(expression_t));
    expression_t * secondExpressionAux = malloc(sizeof(expression_t));

    expressionShallowCopy(firstExpression, firstExpressionAux);
    expressionShallowCopy(secondExpression, secondExpressionAux);

    expression->algebraic_op->op = token;
    expression->algebraic_op->left = firstExpressionAux;
    expression->algebraic_op->right = secondExpressionAux;
    return expression;
}

expression_t * AlgebraicExpressionGrammarAction(algebraic_op_t * algebraicOp) {
    expression_t * expression = malloc(sizeof(expression_t));
    algebraic_op_t * algebraicAux = malloc(sizeof(algebraic_op_t));
    algebraicAux->left = algebraicOp->left;
    algebraicAux->right = algebraicOp->right;
    algebraicAux->op = algebraicOp->op;
    expression->algebraic_op = algebraicAux;
    return expression;
}

expression_t * EvaluateExpressionGrammarAction(evaluate_t * evaluate) {
    expression_t * expression = malloc(sizeof(expression_t));
    evaluate_t * evalAux = malloc(sizeof(evaluate_t));
    evalAux->function = evaluate->function;
    evalAux->value = evaluate->value;
    expression->evaluate = evalAux;
    return expression;
}

expression_t * DerivativeExpressionGrammarAction(derivative_t * derivative) {
    expression_t * expression = malloc(sizeof(expression_t));
    derivative_t * derAux = malloc(sizeof(derivative_t));
    derAux->function = derivative->function;
    derAux->value = derivative->value;
    derAux->error = derivative->error;
    expression->derivative = derAux;
    return expression;
}

expression_t * IntegrateExpressionGrammarAction(integrate_t * integrate) {
    expression_t * expression = malloc(sizeof(expression_t));
    integrate_t * intAux = malloc(sizeof(integrate_t));
    intAux->function = integrate->function;
    intAux->lvalue = integrate->lvalue;
    intAux->rvalue = integrate->rvalue;
    intAux->error = integrate->error;
    expression->integrate = intAux;
    return expression;
}

expression_t * NegativeExpressionGrammarAction(expression_t * negative) {
    expression_t  * expression = malloc(sizeof(expression_t));
    negative_t * negAux = malloc(sizeof(negative_t));
    negAux->expression = negative;
    expression->negative = negAux;
    return expression;
}

expression_t * ParenthesisExpressionGrammarAction(expression_t * exp) {
    expression_t * expression = malloc(sizeof(expression_t));
    parenthesis_exp_t * parAux = malloc(sizeof(parenthesis_exp_t));
    parAux->expression = exp;
    expression->parenthesis_exp = parAux;
    return expression;
}

expression_t * VariableExpressionGrammarAction(variable_t * variable) {
    expression_t * expression = malloc(sizeof(expression_t));
    variable_t * varAux = malloc(sizeof(variable_t));
    varAux->value = variable->value;
    varAux->name = variable->name;
    expression->variable = varAux;
    return expression;
}

statement_t * VariableStatementGrammarAction(variable_t * variable) {
    statement_t * statement = malloc(sizeof(statement_t));
    variable_t * varAux = malloc(sizeof(variable_t));
    varAux->name = variable->name;
    varAux->value = variable->value;
    statement->variable = varAux;
    return statement;
}

variable_t * VariableGrammarAction(char * name, expression_t * expression) {
    variable_t * variable = malloc(sizeof(variable_t));
    expression_t * expAux = malloc(sizeof(expression_t));
    size_t size = strlen(name);
    char * nameAux = malloc(size + 1);

    expressionShallowCopy(expression, expAux);
    strcpy(nameAux, name);

    variable->value = expAux;
    variable->name = nameAux;
    return variable;
}

function_t * TaylorFunctionGrammarAction(taylor_t * taylor) {
    function_t * function = malloc(sizeof(function_t));
    taylor_t * taylorAux = malloc(sizeof(taylor_t));
    taylorAux->function = taylor->function;
    taylorAux->num = taylor->num;
    function->taylor = taylorAux;
    return function;
}

void functionShallowCopy(function_t * src, function_t * dest) {
    dest->taylor = src->taylor;
    dest->expression = src->expression;
    dest->name = src->name;
    dest->composite = src->composite;
    dest->domain = src->domain;
}

composite_t * CompositeIdGrammarAction(id_t * id1, id_t * id2) {
    composite_t * composite = malloc(sizeof(composite_t));
    function_t * funAux1 = malloc(sizeof(function_t));
    function_t * funAux2 = malloc(sizeof(function_t));

    funAux1 = id1->function;
    funAux2 = id2->function;

    composite->inner_funct = funAux1;
    composite->outter_funct = funAux2;
    return composite;
}

composite_t * CompositeCompGrammarAction(composite_t * composite, id_t * id) {
    composite_t * compAux = malloc(sizeof(composite_t));
    function_t * funAux1 = malloc(sizeof(function_t));
    function_t * funAux2 = malloc(sizeof(function_t));
    
    compositeShallowCopy(composite, compAux);

    funAux1->composite = composite;
    funAux2 = id->function;

    composite->inner_funct = funAux1;
    composite->outter_funct = funAux2;
    return composite;
}

domain_t * DomainOneOptOperatorGrammarAction(expression_t * firstExpression, int token, expression_t * secondExpression) {
    domain_t * domain = malloc(sizeof(expression_t));
    expression_t * firstExpressionAux = malloc(sizeof(expression_t));
    expression_t * secondExpressionAux = malloc(sizeof(expression_t));

    expressionShallowCopy(firstExpression, firstExpressionAux);
    expressionShallowCopy(secondExpression, secondExpressionAux);

    domain->domain_one->left = firstExpressionAux;
    domain->domain_one->right = secondExpressionAux;
    domain->domain_one->oper = token;

    return domain;
}

domain_t * DomainTwoOptOperatorGrammarAction(expression_t * firstExpression, int token1, expression_t * secondExpression, int token2, expression_t * thirdExpression) {
    domain_t * domain = malloc(sizeof(expression_t));
    expression_t * firstExpressionAux = malloc(sizeof(expression_t));
    expression_t * secondExpressionAux = malloc(sizeof(expression_t));
    expression_t * thirdExpressionAux = malloc(sizeof(expression_t));

    expressionShallowCopy(firstExpression, firstExpressionAux);
    expressionShallowCopy(secondExpression, secondExpressionAux);
    expressionShallowCopy(thirdExpression, thirdExpressionAux);

    domain->domain_two->left = firstExpressionAux;
    domain->domain_two->mid = secondExpressionAux;
    domain->domain_two->right = thirdExpression;
    domain->domain_two->oper1 = token1;
    domain->domain_two->oper2 = token2;

    return domain;
}

boolean_t * DomainGrammarAction(domain_t * domain) {
    boolean_t * boolean = malloc(sizeof(boolean_t));
    domain_t * domainAux = malloc(sizeof(domain_t));

    domainAux->domain_one = domain->domain_one;
    domainAux->domain_two = domain->domain_two;
    
    boolean->domain = domainAux;
    
    return boolean;

}

void booleanShallowCopy(boolean_t * source, boolean_t * dest) {
    dest->domain = source->domain;
    dest->bool_op = source->bool_op;
    dest->not_bool = source->not_bool;
}

boolean_t * BooleanOperatorGrammarAction(boolean_t * left, int token, boolean_t * right) {
    boolean_t * boolean = malloc(sizeof(boolean_t));
    boolean_t * leftAux = malloc(sizeof(boolean_t));
    boolean_t * rightAux = malloc(sizeof(boolean_t));

    booleanShallowCopy(left, leftAux);
    booleanShallowCopy(right, rightAux);
    
    boolean->bool_op->left = leftAux;
    boolean->bool_op->right = rightAux;
    boolean->bool_op->op = token;
    
    return boolean;

}

boolean_t * NotBooleanAction(boolean_t * notBoolean) {
    boolean_t * boolean = malloc(sizeof(boolean_t));
    not_bool_t * notBooleanAux = malloc(sizeof(not_bool_t));

    booleanShallowCopy(notBoolean, notBooleanAux->boolean);
    
    boolean->not_bool = notBooleanAux;

    return boolean;
}

function_t * DeclareFunctionGrammarAction(id_t * id, function_t * func) {
    function_t * function = malloc(sizeof(function_t));
    function_t * funcAux = malloc(sizeof(function_t));

    functionShallowCopy(func, function);

    function->name = id;

    return function;
}

function_t * DeclareFunctionListGrammarAction(id_t * id, function_t * func) {
    function_t * function = malloc(sizeof(function_t));

    functionShallowCopy(func, function);

    function->name = id;
    
    return function;
}

function_t * FunctionsGrammarAction(function_t * func1, function_t * func2) {
    function_t * funcFinal = malloc(sizeof(function_t));
    function_t * funcAux = malloc(sizeof(function_t));

    functionShallowCopy(func1, funcFinal);
    functionShallowCopy(func2, funcAux);
    
    funcFinal->next = funcAux;
    return funcFinal;
}

function_t * CompositeFunctionGrammarAction(composite_t * composite) {
    function_t * function = malloc(sizeof(function_t));
    composite_t * compAux = malloc(sizeof(composite_t));

    compositeShallowCopy(composite, compAux);

    function->composite = compAux;
    return function;
}

function_t * ExpressionFunctionGrammarAction(expression_t * expression) {
    function_t * function = malloc(sizeof(function_t));
    expression_t * expAux = malloc(sizeof(expression_t));

    expressionShallowCopy(expression, expAux);
    function->expression = expAux;
    return function;
}

void domainShallowCopy(domain_t * src, domain_t * dest) {
    dest->domain_one = src->domain_one;
    dest->domain_two = src->domain_two;
}

function_t * ExpressionDomainFunctionGrammarAction(expression_t * expression, domain_t * domain) {
    function_t * function = malloc(sizeof(function_t));
    expression_t * expAux = malloc(sizeof(expression_t));
    domain_t * domAux = malloc(sizeof(domain_t));

    expressionShallowCopy(expression, expAux);
    domainShallowCopy(domain, domAux);
    function->expression = expAux;
    function->domain = domAux;
    return function;
}

expression_t * ExpressionsGrammarAction(expression_t * exp1, expression_t * exp2) {
    expression_t * expFinal = malloc(sizeof(expression_t));
    expression_t * expAux = malloc(sizeof(expression_t));

    expressionShallowCopy(exp1, expFinal);
    expressionShallowCopy(exp2, expAux);
    
    expFinal->next = expAux;
    return expFinal;
}

print_t * PrintExpression(expression_t * expression) {
    print_t * print = malloc(sizeof(print_t));
    expression_t * expAux = malloc(sizeof(expression_t));

    expressionShallowCopy(expAux, expression);
    
    print->exp = expAux;
    return print;
}

print_t * PrintExpression(char * string) {
    print_t * print = malloc(sizeof(print_t));
    char * strAux = malloc(strlen(string));

    strcpy(strAux, string);
    
    print->string = strAux;
    return print;
}

statement_t * StatementsGrammarAction(statement_t * stat1, statement_t * stat2) {
    statement_t * statFinal = malloc(sizeof(statement_t));
    statement_t * statAux = malloc(sizeof(statement_t));

    statementShallowCopy(stat1, statFinal);
    statementShallowCopy(stat2, statAux);
    
    statFinal->next = statAux;
    return statFinal;
}

void booleanShallowCopy(boolean_t * src, boolean_t * dest) {
    dest->domain = src->domain;
    dest->not_bool = src->not_bool;
    dest->bool_op = src->bool_op;
}

void blockShallowCopy(block_t * src, block_t * dest) {
    dest->statement = src->statement;
}

if_t * IfBlockGrammarAction(boolean_t * boolean, block_t * block) {
    if_t * if_st = malloc(sizeof(if_t));
    boolean_t * boolAux = malloc(sizeof(boolean_t));
    block_t * blockAux = malloc(sizeof(block_t));

    booleanShallowCopy(boolean, boolAux);
    blockShallowCopy(block, blockAux);

    if_st->boolean = boolAux;
    if_st->block = blockAux;
    return if_st;
}

if_t * IfBlockGrammarAction(boolean_t * boolean, block_t * block) {
    if_t * if_st = malloc(sizeof(if_t));
    boolean_t * boolAux = malloc(sizeof(boolean_t));
    block_t * blockAux = malloc(sizeof(block_t));

    booleanShallowCopy(boolean, boolAux);
    blockShallowCopy(block, blockAux);

    if_st->boolean = boolAux;
    if_st->block = block;
    return if_st;
}

if_t * IfElseBlockGrammarAction(boolean_t * boolean, block_t * block, block_t * else_block) {
    if_t * if_st = malloc(sizeof(if_t));
    boolean_t * boolAux = malloc(sizeof(boolean_t));
    block_t * blockAux = malloc(sizeof(block_t));
    block_t * elseBlockAux = malloc(sizeof(block_t));

    booleanShallowCopy(boolean, boolAux);
    blockShallowCopy(block, blockAux);
    blockShallowCopy(else_block, elseBlockAux);

    if_st->boolean = boolAux;
    if_st->block = blockAux;
    if_st->else_block = elseBlockAux;
    return if_st;
}

while_t * WhileBlockGrammarAction(boolean_t * boolean, block_t * block) {
    while_t * while_st = malloc(sizeof(while_t));
    boolean_t * boolAux = malloc(sizeof(boolean_t));
    block_t * blockAux = malloc(sizeof(block_t));

    booleanShallowCopy(boolean, boolAux);
    blockShallowCopy(block, blockAux);

    while_st->boolean = boolAux;
    while_st->block = block;
    return while_st;
}