%{

#include "bison-actions.h"
#include "nodes.h"

%}

%union {
    variable_t * variable;
    function_t * function;
    statement_t * statement;
    expression_t * expression;
    derivative_t * derivative;
    evaluate_t * evaluate;
    integrate_t * integrate;
    algebraic_op_t * algebraic_op;
    parenthesis_exp_t * parenthesis_exp;
    boolean_t * boolean;
    domain_t * domain;
    int intNumber;
    double doubleNumber;
    char * string;
    int token;
}

%type <variable> variable
%type <variable> declare_variable
%type <function> function
%type <function> functions
%type <function> declare_function
%type <statement> statement
%type <expression> expression
%type <derivative> derivative
%type <integrate> integrate
%type <evaluate> evaluate
%type <algebraic_op> algebraic_op
%type <parenthesis_exp> parenthesis_exp
%type <boolean> boolean
%type <domain> domain

%token <token> ADD
%token <token> SUB
%token <token> MUL
%token <token> DIV
%token <token> POW
%token <token> FUNCTION
%token <token> IF
%token <token> ELSE
%token <token> FOR
%token <token> WHILE
%token <token> PRINT
%token <token> SEMICOLON
%token <token> COMPOSITE
%token <token> SQUOTE
%token <token> DERIVATIVE
%token <token> INTEGRATE
%token <token> BETWEEN
%token <token> AND_INT
%token <token> EVALUATE
%token <token> IN
%token <token> OPEN_BRACES
%token <token> OPEN_BRACKETS
%token <token> CLOSE_BRACES
%token <token> CLOSE_BRACKETS
%token <token> OPEN_PARENTHESIS
%token <token> CLOSE_PARENTHESIS
%token <token> ASSIGN
%token <token> EQUAL
%token <token> AND
%token <token> OR
%token <token> NOT
%token <token> ADD_ASSIGN
%token <token> INCREMENT
%token <token> SUB_ASSIGN
%token <token> DECREMENT
%token <token> MUL_ASSIGN
%token <token> DIV_ASSIGN
%token <string> DQUOTE
%token <token> COMMA
%token <token> COLON
%token <token> LESSER
%token <token> LESSER_EQUAL
%token <token> GREATER
%token <token> GREATER_EQUAL
%token <token> NOT_EQUAL
%token <token> VAR
%token <intNumber> ERR
%token <token> TAYLOR
%token <string> ESCAPESTRING
%token <doubleNumber> DOUBLE
%token <intNumber> INTEGER
%token <string> ID

%precedence CLOSE_PARENTHESIS
%precedence ELSE

%left GREATER GREATER_EQUAL LESSER LESSER_EQUAL EQUAL NOT_EQUAL
%left ADD SUB
%right NOT
%left MUL DIV AND OR
%left POW
%left SQUOTE

%%

program: statements                                                                 { $$ = ProgramGrammarAction($1); }
    ;

statement: evaluate SEMICOLON                                                       { $$ = EvaluateGrammarAction($1); }
    | integrate SEMICOLON                                                           { $$ = IntegrateGrammarAction($1); }
    | derivative SEMICOLON                                                          { $$ = DerivativeGrammarAction($1); }
    | declare_function SEMICOLON                                                    { $$ = StatementDeclareFunctionGrammarAction($1); }
    | if_statement                                                                  { $$ = IfGrammarAction($1); }
    | for_statement                                                                 { $$ = ForGrammarAction($1); }
    | while_statement                                                               { $$ = WhileGrammarAction($1); }
    | print SEMICOLON                                                               { $$ = PrintGrammarAction($1); }
    | declare_variable SEMICOLON                                                    { $$ = VariableStatementGrammarAction($1); }
    | assign_variable SEMICOLON                                                     { $$ = VariableStatementGrammarAction($1); }
    ;

expression: positive_constant                                                       { $$ = PositiveConstantGrammarAction($1) }
    | evaluate                                                                      { $$ = EvaluateGrammarAction($1) }
    | integrate                                                                     { $$ = IntegrateGrammarAction($1) }
    | derivative                                                                    { $$ = DerivativeGrammarAction($1) }
    | ID                                                                            { $$ = IdGrammarAction($1) }
    | expression ADD expression                                                     { $$ = ExpressionOperatorGrammarAction($1, $2, $3) }
    | expression SUB expression                                                     { $$ = ExpressionOperatorGrammarAction($1, $2, $3) }
    | expression MUL expression                                                     { $$ = ExpressionOperatorGrammarAction($1, $2, $3) }
    | expression DIV expression                                                     { $$ = ExpressionOperatorGrammarAction($1, $2, $3) }
    | expression POW expression                                                     { $$ = ExpressionOperatorGrammarAction($1, $2, $3) }
    | OPEN_PARENTHESIS expression CLOSE_PARENTHESIS                                 { $$ = ParenthesisExpressionGrammarAction($2) }
    | SUB expression                                                                { $$ = NegativeExpressionGrammarAction($2) }
    ;

domain: expression LESSER expression                                                { $$ = DomainOneOptOperatorGrammarAction($1, $2, $3) }
    | expression LESSER_EQUAL expression                                            { $$ = DomainOneOptOperatorGrammarAction($1, $2, $3) }
    | expression GREATER_EQUAL expression                                           { $$ = DomainOneOptOperatorGrammarAction($1, $2, $3) }
    | expression GREATER expression                                                 { $$ = DomainOneOptOperatorGrammarAction($1, $2, $3) }
    | expression EQUAL expression                                                   { $$ = DomainOneOptOperatorGrammarAction($1, $2, $3) }
    | expression NOT_EQUAL expression                                               { $$ = DomainOneOptOperatorGrammarAction($1, $2, $3) }
    | expression LESSER expression LESSER expression                                { $$ = DomainTwoOptOperatorGrammarAction($1, $2, $3, $4, $5) }
    | expression LESSER_EQUAL expression LESSER expression                          { $$ = DomainTwoOptOperatorGrammarAction($1, $2, $3, $4, $5) }
    | expression LESSER expression LESSER_EQUAL expression                          { $$ = DomainTwoOptOperatorGrammarAction($1, $2, $3, $4, $5) }
    | expression LESSER_EQUAL expression LESSER_EQUAL expression                    { $$ = DomainTwoOptOperatorGrammarAction($1, $2, $3, $4, $5) }
    ;

boolean: domain                                                                     { $$ = DomainGrammarAction($1) }
    | boolean OR boolean                                                            { $$ = BooleanOperatorGrammarAction($1, $2, $3) }
    | boolean AND boolean                                                           { $$ = BooleanOperatorGrammarAction($1, $2, $3) }
    | OPEN_PARENTHESIS boolean CLOSE_PARENTHESIS                                    { $$ = $2 }
    | NOT boolean                                                                   { $$ = NotBooleanAction($2) }
    ;

positive_constant: DOUBLE                                                           { $$ = DoubleGrammarAction($1) }
    | INTEGER                                                                       { $$ = IntegerGrammarAction($1) }
    ;

declare_function: FUNCTION ID ASSIGN function                                       { $$ = DeclareFunctionGrammarAction($2, $4) }
    | FUNCTION ID ASSIGN OPEN_BRACES functions CLOSE_BRACES                         { $$ = DeclareFunctionListGrammarAction($2, $5) }
    ;

function: expression                                                                { $$ = ExpressionFunctionGrammarAction($1) }
    | expression COLON domain                                                       { $$ = ExpressionDomainFunctionGrammarAction($1, $3) }
    | composite                                                                     { $$ = CompositeFunctionGrammarAction($1) }
    | taylor                                                                        { $$ = TaylorFunctionGrammarAction($1) }
    ;

functions: function                                                                 { $$ = $1 }
    | function COMMA functions                                                      { $$ = FunctionsGrammarAction($1, $3) }
    ;

composite: ID COMPOSITE ID                                                          { $$ = CompositeIdGrammarAction($1, $3) }
    | composite COMPOSITE ID                                                        { $$ = CompositeCompGrammarAction($1, $3) }
    ;

expressions: expression                                                             { $$ = $1 }
    | expression COMMA expressions                                                  { $$ = ExpressionsGrammarAction($1, $3) }
    ;

ids: ID                                                                             { $$ = $1 }
    | ID COMMA ids                                                                  { $$ = DoubleGrammarAction($1) }
    ;

numArray: OPEN_BRACKETS expressions CLOSE_BRACKETS                                  { $$ = DoubleGrammarAction($1) }
    ;

numArrays: numArray                                                                 { $$ = DoubleGrammarAction($1) }
    | numArray COMMA numArrays                                                      { $$ = DoubleGrammarAction($1) }
    ;

arrayNumArray: OPEN_BRACKETS numArrays CLOSE_BRACKETS                               { $$ = DoubleGrammarAction($1) }
    ;

idArray: OPEN_BRACKETS ids CLOSE_BRACKETS                                           { $$ = DoubleGrammarAction($1) }
    ;

escapeString: ESCAPESTRING                                                          { $$ = DoubleGrammarAction($1) }
    | ESCAPESTRING ADD escapeString                                                 { $$ = DoubleGrammarAction($1) }
    ;

escapeStrings: escapeString                                                         { $$ = $1 }
    | escapeString COMMA escapeStrings                                              { $$ = DoubleGrammarAction($1) }
    ;

escapeStringArray: OPEN_BRACKETS escapeStrings CLOSE_BRACKETS                       { $$ = DoubleGrammarAction($1) }
    ;

evaluate: EVALUATE ID IN OPEN_PARENTHESIS expression CLOSE_PARENTHESIS              { $$ = DoubleGrammarAction($1) }
    | EVALUATE ID IN numArray                                                       { $$ = DoubleGrammarAction($1) }
    | ID OPEN_PARENTHESIS expression CLOSE_PARENTHESIS                              { $$ = DoubleGrammarAction($1) }
    | EVALUATE idArray IN OPEN_PARENTHESIS expression CLOSE_PARENTHESIS             { $$ = DoubleGrammarAction($1) }
    | EVALUATE idArray IN numArray                                                  { $$ = DoubleGrammarAction($1) }
    | idArray OPEN_PARENTHESIS expression CLOSE_PARENTHESIS                         { $$ = DoubleGrammarAction($1) }
    ;

integrate: INTEGRATE ID BETWEEN OPEN_PARENTHESIS expression AND_INT expression CLOSE_PARENTHESIS                     { $$ = DoubleGrammarAction($1) }
    | INTEGRATE ID OPEN_PARENTHESIS expression AND_INT expression CLOSE_PARENTHESIS                                  { $$ = DoubleGrammarAction($1) }
    | INTEGRATE idArray BETWEEN OPEN_PARENTHESIS expression AND_INT expression CLOSE_PARENTHESIS                     { $$ = DoubleGrammarAction($1) }
    | INTEGRATE idArray OPEN_PARENTHESIS expression AND_INT expression CLOSE_PARENTHESIS                             { $$ = DoubleGrammarAction($1) }
    | integrate ERR positive_constant                                                                                { $$ = DoubleGrammarAction($1) }
    ;

squotes: SQUOTE                                                                     { $$ = DoubleGrammarAction($1) }
    | SQUOTE squotes                                                                { $$ = DoubleGrammarAction($1) }
    ;

print: PRINT OPEN_PARENTHESIS expression CLOSE_PARENTHESIS                          { $$ = PrintExpression($3) }
    | PRINT OPEN_PARENTHESIS escapeString CLOSE_PARENTHESIS                         { $$ = PrintExpression($3) }
    ;

statements: statement                                                               { $$ = $1 }
    | statement statements                                                          { $$ = StatementsGrammarAction($1, $2) }
    ;

derivative: DERIVATIVE ID IN OPEN_PARENTHESIS expression CLOSE_PARENTHESIS          { $$ = DoubleGrammarAction($1) }
    | DERIVATIVE INTEGER ID IN OPEN_PARENTHESIS expression CLOSE_PARENTHESIS        { $$ = DoubleGrammarAction($1) }
    | ID squotes OPEN_PARENTHESIS expression CLOSE_PARENTHESIS                      { $$ = DoubleGrammarAction($1) }
    | DERIVATIVE idArray IN OPEN_PARENTHESIS expression CLOSE_PARENTHESIS           { $$ = DoubleGrammarAction($1) }
    | DERIVATIVE INTEGER idArray IN OPEN_PARENTHESIS expression CLOSE_PARENTHESIS   { $$ = DoubleGrammarAction($1) }
    | idArray squotes OPEN_PARENTHESIS expression CLOSE_PARENTHESIS                 { $$ = DoubleGrammarAction($1) }
    | derivative ERR positive_constant                                              { $$ = DoubleGrammarAction($1) }
    ;

block: OPEN_BRACES statements CLOSE_BRACES                                          { $$ = DoubleGrammarAction($1) }
    | statement                                                                     { $$ = DoubleGrammarAction($1) }
    ;

if_statement: IF OPEN_PARENTHESIS boolean CLOSE_PARENTHESIS block                   { $$ = IfBlockGrammarAction($3, $5) }
    | IF OPEN_PARENTHESIS boolean CLOSE_PARENTHESIS block ELSE block                { $$ = IfElseBlockGrammarAction($3, $5, $7) }
    ;

for_statement: FOR OPEN_PARENTHESIS declare_variable SEMICOLON boolean SEMICOLON assign_variable CLOSE_PARENTHESIS block        { $$ = DoubleGrammarAction($1) }
    | FOR OPEN_PARENTHESIS SEMICOLON boolean SEMICOLON assign_variable CLOSE_PARENTHESIS block                                  { $$ = DoubleGrammarAction($1) }
    ;

taylor: TAYLOR INTEGER ID                                                           { $$ = DoubleGrammarAction($1) }
    ;

while_statement: WHILE OPEN_PARENTHESIS boolean CLOSE_PARENTHESIS block             { $$ = WhileBlockGrammarAction($3, $5) }
    ;

declare_variable: VAR ID                                                            { $$ = DoubleGrammarAction($1) }
    | declare_variable ASSIGN expression                                            { $$ = DoubleGrammarAction($1) }
    ;

assign_variable: ID ADD_ASSIGN expression                                           { $$ = DoubleGrammarAction($1) }
    | ID SUB_ASSIGN expression                                                      { $$ = DoubleGrammarAction($1) }
    | ID MUL_ASSIGN expression                                                      { $$ = DoubleGrammarAction($1) }
    | ID DIV_ASSIGN expression                                                      { $$ = DoubleGrammarAction($1) }
    | ID INCREMENT                                                                  { $$ = DoubleGrammarAction($1) }
    | ID DECREMENT                                                                  { $$ = DoubleGrammarAction($1) }
    | ID ASSIGN expression                                                          { $$ = DoubleGrammarAction($1) }
    ;

%%
